# -*- coding: utf-8 -*-

#  icd_bolt_set.py    2019.10.13  kita
#!!  ボルト配置補助 Script

# 関係ファイル
#   icd_bolt_set.py  my_vector.py  numpy(要Install)  icd_menu.py3  bolt_set.icd
#   icd_bolt_set.ini or icd_bolt_setJ.ini(iCAD標準ボルト) or icd_bolt_set_sby.ini(SBY標準ボルト)

# 起動方法 PowerShellもしくはComand Promptより
# py .\icd_bolt_set.py3 icd_bolt_set.ini or .\icd_bolt_set.py3 icd_bolt_set.ini (環境により使い分ける)

# 参考  SxFileMachinePart\Open\Open.cs
#       SxEntSeg\GetInfHoleList\GetInfHoleList.cs

# 関数一覧
# def get_ini(target_fn): # ボルト類のパラメータファイル名を外部ファイルより取得
# def set_jis_len():    # ボルトのJIS規格長さを取得 配列 lj
# def menu_bolt():    #ボルトの仕様を設定するメニュー
# def menu_edge():  # Edge 選択(キリ穴部品、タップ部品のhole_listを取得)
# def get_bolt_list(hole_list,bolt_spec): # 配置ボルトのリスト作成  bolt_list取得
# def bolt_set(bolt_list, bolt_spec, fn_dic, lj):  # JIS部品(ボルト)をパーツ化して配置する。
# def _main():  # メインルーチン

# 履歴
#19.11.02   不完全ネジ部の長さを指定(薄板にも対応)
#19.11.03   共締め部品へ対応    
#19.11.04   ボルト類のファイル指定を外部ファイル化 icd_bolt_set.ini
#           "エッジ選択"エラー対応
#19.11.05   bolt長さをJIS規格化
#19.11.08   連続貼り付けモード追加(Default)
#19.11.16   各セクションを関数化。ボルト長さにワッシャの厚みをプラス。エッジの連続選択化
#           menu_boltのキー入力をキーイベント化
#20.6.29    Ver.1.10    menuをtkinter化。
#           設定ファイル(icd_bolt_set.ini)を起動スクリプトの引数渡しに変更しました。
#20.8.21    Ver.1.12    キリ穴検索アルゴリズム改良（共締め部品に自動で対応)
#           ボルトが反転したり反対面の同軸袋タップにまでボルトを配置するのを修正
#           割締め部品にも対応　部品選択して右クリック
#20.10.3    「穴属性」にチェック付加
#22.5.6     ワッシャ、SPワッシャをボルト配下に配置（階層）
#22.6.13    長穴対応。ベクトルが直交しない問題対策
#23.7.8     候補２の機械パーツ（備品外）追加 
#           干渉チェック追加
#24.4.2     干渉チェック機能をチェックボタンで選択方式に変更
#24.4.30    ねじ込み深さを指示できるように変更　blt_njkm


import clr    # 必要なアセンブリを読み込み，.NETの名前空間をPythonのパッケージとして扱う
clr.AddReference('sxnet')       # iCAD操作用クラスのdllをimport
from sxnet import *         # iCAD/SX 全てのクラスをimport

import tkinter.ttk as ttk 
import tkinter as tk  
import math                     # 三角関数
import sys
import my_vector as myv
import numpy as np
from numpy import linalg as LA
import os
#import msvcrt           # key event
import time
import my_module as mmd

#--------------------------------------------------------------------
#   共有変数
#--------------------------------------------------------------------
blt_njkm = 1.2      # boltねじ込み長さ係数
gosa = 0.1          # 誤差(キリ穴とタップの許容ずれ)

def get_ini(target_fn): # ボルト類のパラメータファイル名を外部ファイルより取得 
    global size_hex_sus, size_hex_ss, size_cs_sus, size_cs_ss   # 各種ボルトのサイズリスト

    dir_name = os.path.dirname(os.path.abspath(__file__))
    full_path_fn = dir_name + '\\' + target_fn
  
    fp = open(full_path_fn)
    lines = fp.readlines()
    fp.close()
  
    #fn_dic = {}
    fn_blt_hex_sus= []; fn_sw_hex_sus= []; fn_hw_hex_sus= []
    fn_blt_hex_ss= [];  fn_sw_hex_ss= [];  fn_hw_hex_ss= []
    fn_blt_cs_sus= []; fn_sw_cs_sus= []; fn_hw_cs_sus= []
    fn_blt_cs_ss= [];  fn_sw_cs_ss= [];  fn_hw_cs_ss= []
    for line in lines:
        if line.find('#') > -1:   # '#'を含む行はスキップ
            continue
        
        line = line.rstrip()               # 行末の改行を削除
        tmp = line.split(',')
        if tmp[0].find('fn_blt_hex_sus')> -1:
            fn_blt_hex_sus.append(tmp[1])
        elif tmp[0].find('fn_sw_hex_sus')> -1:
            fn_sw_hex_sus.append(tmp[1])
        elif tmp[0].find('fn_hw_hex_sus')> -1:
            fn_hw_hex_sus.append(tmp[1])
        elif tmp[0].find('fn_blt_hex_ss')> -1:
            fn_blt_hex_ss.append(tmp[1])
        elif tmp[0].find('fn_sw_hex_ss')> -1:
            fn_sw_hex_ss.append(tmp[1])
        elif tmp[0].find('fn_hw_hex_ss')> -1:
            fn_hw_hex_ss.append(tmp[1])

        elif tmp[0].find('fn_blt_cs_sus')> -1:
            fn_blt_cs_sus.append(tmp[1])
        elif tmp[0].find('fn_sw_cs_sus')> -1:
            fn_sw_cs_sus.append(tmp[1])
        elif tmp[0].find('fn_hw_cs_sus')> -1:
            fn_hw_cs_sus.append(tmp[1])
        elif tmp[0].find('fn_blt_cs_ss')> -1:
            fn_blt_cs_ss.append(tmp[1])
        elif tmp[0].find('fn_sw_cs_ss')> -1:
            fn_sw_cs_ss.append(tmp[1])
        elif tmp[0].find('fn_hw_cs_ss')> -1:
            fn_hw_cs_ss.append(tmp[1])

    # sizeリスト作成
    size_hex_sus= []; size_hex_ss= []; size_cs_sus= []; size_cs_ss= []
    k= 0
    for fp_fn in fn_blt_hex_sus:
        #print(fp_fn)    ####
        blt_parame = SxFileMachinePart(fp_fn)      # Bolt File パラメータ
        blt_pattern = blt_parame.getPatternNames()     # M3,M4...Sxリスト
        tmp= []
        for i in range(len(blt_pattern)):
            tmp.append(blt_pattern[i])
        size_hex_sus.append(tmp)
            
    for fp_fn in fn_blt_hex_ss:
        #print(fp_fn)    ####
        blt_parame = SxFileMachinePart(fp_fn)      # Bolt File パラメータ
        blt_pattern = blt_parame.getPatternNames()     # M3,M4...Sxリスト
        tmp= []
        for i in range(len(blt_pattern)):
            tmp.append(blt_pattern[i])
        size_hex_ss.append(tmp)

    for fp_fn in fn_blt_cs_sus:
        #print(fp_fn)    ####
        blt_parame = SxFileMachinePart(fp_fn)      # Bolt File パラメータ
        blt_pattern = blt_parame.getPatternNames()     # M3,M4...Sxリスト
        tmp= []
        for i in range(len(blt_pattern)):
            tmp.append(blt_pattern[i])
        size_cs_sus.append(tmp)

    for fp_fn in fn_blt_cs_ss:
        #print(fp_fn)    ####
        blt_parame = SxFileMachinePart(fp_fn)      # Bolt File パラメータ
        blt_pattern = blt_parame.getPatternNames()     # M3,M4...Sxリスト
        tmp= []
        for i in range(len(blt_pattern)):
            tmp.append(blt_pattern[i])
        size_cs_ss.append(tmp)
    
    #print(fn_blt_hex_sus)
    #print(size_hex_sus)
    #print(size_hex_ss)
    #print(size_cs_sus)
    #print(size_cs_ss)   #####

    return (fn_blt_hex_sus, fn_sw_hex_sus, fn_hw_hex_sus, fn_blt_hex_ss, fn_sw_hex_ss, fn_hw_hex_ss, \
            fn_blt_cs_sus, fn_sw_cs_sus, fn_hw_cs_sus, fn_blt_cs_ss, fn_sw_cs_ss, fn_hw_cs_ss)


def get_ini_bak(target_fn): # ボルト類のパラメータファイル名を外部ファイルより取得 

    dir_name = os.path.dirname(os.path.abspath(__file__))
    full_path_fn = dir_name + '\\' + target_fn
  
    fp = open(full_path_fn)
    lines = fp.readlines()
    fp.close()
  
    fn_dic = {}
    for line in lines:
        if line.find('#') > -1:   # '#'を含む行はスキップ
            continue
        if len(line) > 5:
            line = line.rstrip()               # 行末の改行を削除
            tmp = line.split(',')
            fn_dic[tmp[0]] = tmp[1]
            #print(tmp[0] + '  ' + tmp[1])

    # sizeリスト作成


    return fn_dic

# end of get_ini()

def set_jis_len():    # ボルトのJIS規格長さを取得 配列 lj
  lj = []
  # JIS規格ボルト長さのの配列-------------------------------------------------------------------------
  # 配列ljのindexにボルト長さを入れるとJIS長さが求まる
             #0,1,2,3,4,5,6,7,8, 9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30
  lj[0:30] = [5,5,5,5,5,5,6,8,8,10,10,12,12,14,14,16,16,18,18,20,20,22,22,25,25,25,28,28,28,30,30]
              #31,32,33,34,35,36,37,38,39,40
  lj[31:40] = [32,32,35,35,35,38,38,38,40,40]

  lj[41:45] = [45] * 5; lj[46:50] = [50] * 5; lj[51:55] = [55] * 5; lj[56:60] = [60] * 5
  lj[61:65] = [65] * 5; lj[66:70] = [70] * 5; lj[71:75] = [75] * 5; lj[75:80] = [80] * 5
  lj[81:85] = [85] * 5; lj[86:90] = [90] * 5; lj[91:95] = [95] * 5; lj[96:100] = [100] * 5
  lj[101:105] = [105] * 5; lj[106:110] = [110] * 5; lj[111:115] = [115] * 5; lj[116:120] = [120] * 5
  #----------------------------------------------------------------------------------------------------
  return lj


# 円弧のエッジがタップ中心と一致するかどうか
def is_tup(edge, holes):
    geom = edge.getGeom()
    if geom.type != SxGeom.TYPE_CIRCLE3D:
        return None, None, None
    #print(geom.type )   #!!!
    cp0 = np.array([geom.cp.x, geom.cp.y, geom.cp.z])   # 3D円弧の中心座標
    #print(cp0)  #!!!
    for hole in holes:
        if hole.name.find("タップ穴") > -1:
            cp1 = np.array([hole.cs.org.x, hole.cs.org.y, hole.cs.org.z])
            v0 = np.array([hole.cs.zvec.x, hole.cs.zvec.y, hole.cs.zvec.z]) # 穴方向Vector
            h = hole.param_list[2].value  # 穴深さ float
            cp2 = cp1 + h*v0 # タップ底中心Vector
            #if cp0.all() == cp1.all():
            if cp0[0] == cp1[0] and cp0[1] == cp1[1] and cp0[2] == cp1[2] :
                #print('     タップです'); print(cp1)  #!!!
                pm = 1
                return (hole, cp0, pm)
            elif cp0[0] == cp2[0] and cp0[1] == cp2[1] and cp0[2] == cp2[2] :
                #print('     タップです'); print(cp2)  #!!!
                pm = -1
                return (hole, cp0, pm)
        else:
            continue
    return None, None, None

# tupとエッジ情報よりボルト配置
def bolt_set2(tup, cpt, edge, pm):
    bolt_spec = get_bolt_spec()
    
    print('\n    長穴ボルトセット開始!') #!!!

    #キリ穴深さ hkr
    v0 = np.array([tup.cs.zvec.x, tup.cs.zvec.y, tup.cs.zvec.z]) # 穴方向Vector
    #p0 = np.array([tup.cs.org.x, tup.cs.org.y, tup.cs.org.z]) #穴中心
    geom = edge.getGeom()
    if geom.type == SxGeom.TYPE_CIRCLE3D or geom.type == SxGeom.TYPE_ARC3D:   # int 67, 68
        p1 = np.array([geom.cp.x, geom.cp.y, geom.cp.z])
    elif geom.type == SxGeom.TYPE_LINE3D:   # int 65
        p1 = np.array([geom.sp.x, geom.sp.y, geom.sp.z])

    hkr = myv.f2plen(v0, cpt, p1)
    #print(hkr)  #!!!
    # タップの仕様
    d = tup.param_list[0].value  # 外径  float
    htp = tup.param_list[2].value  # 穴深さ float
    kantu =  tup.penetrate       # 貫通属性　True False
    v_cp0 = np.array([tup.cs.org.x, tup.cs.org.y, tup.cs.org.z]) #穴中心
    v_vec = np.array([tup.cs.zvec.x, tup.cs.zvec.y, tup.cs.zvec.z]) # 穴方向
    v_cp1 = v_cp0 + htp * v_vec     # 穴底中心

    if pm == 1:
        tmp_v = v_cp0 + hkr*(-v_vec)
        kr_list = [[d*1.1, hkr, tmp_v, v_cp0, -v_vec, 'k']] 
    else:
        tmp_v = v_cp1 + hkr*(v_vec)
        kr_list = [[d*1.1, hkr, v_cp1, tmp_v, -v_vec, 'k']] 
        
    tp = [d, htp, v_cp0, v_cp1, v_vec, kantu]

    (blt_len, blt_cp, blt_vec, blt_b, kr) = get_blt_cp2(tp, kr_list)
    
    # ボルト仕様
    blt_size = d
    blt_type =  bolt_spec[0];  blt_zai = bolt_spec[1]; blt_sw = bolt_spec[2]
    blt_hw =  bolt_spec[3]; 
    blt = [blt_size, blt_len, blt_cp, blt_vec, blt_type, blt_sw, blt_hw, blt_b] 

    bolt_set([blt])

def menu_edge():  # Edge 選択  (キリ穴部品、タップ部品のhole_listを取得)
    #global loop_edge
    #loop_edge = True
    opt = SxOptSelect()
    opt.ent_mode = SxSys.ENTMODE_SEG
    #opt.select_ent = True
    opt.select_pos = True
    opt.select_go = True
    # エッジ選択を有効にするには、この行のコメントを外す。
    #opt.select_edge = True
    #opt.select_face = True # フェイス選択を有効にするには、この行のコメントを外す。
  
    menu2 = True
    #menu3 = True
    idx = 1     # 部品No
    hole_list2 = []      #穴とタップのリストのリスト
    tup = None
    while menu2:
        if idx == 1:
            print("\n\n  部品 %d のエッジをピックして下さい。(GOで終了)" % idx)
        else:
            print("\n\n  部品 %d のエッジをピックして下さい。(GOで割締部品)" % idx)

        try:    # 他のコマンド選択された場合などでもエラー回避
            inf = SxSys.select(opt)
            # GOなら 19.11.09 追加
            if inf.status == SxInfSelect.STATUS_GO:
                menu2 = False
                if idx == 2:
                    print('\n     割締め部品と判断しました\n')
                break   # menu2を抜ける len(hole_list2) = 1

            else:
                edge = inf.hit_edge1
                #print(edge) #!!!
                #エッジが属するエンティティを問い合わせる
                ent = edge.getEntSeg()
                # ID取得
                ID = ent.ID
                #print(ID)   #!!!
                if idx == 1:    # 1個めのピック
                    pre_ID = ID
                else:           # 2個めのピック    
                    if ID == pre_ID:
                        print('\n   同一部品です。\007\n')
                        continue

                # キリ穴情報の取得
                #hole_list.append(ent.getInfHoleList())
                holes = ent.getInfHoleList()
                
                #print(holes)    #!!!
                if holes == None and tup == None:   
                    print("\n\n  指定要素にキリ穴、タップはありません。\007 ")
                    continue
                else:
                    if idx == 1:
                        # エッジがタップの一部かどうか
                        (tup, cpt, pm) = is_tup(edge, holes)
                        if tup != None: # タップのエッジ
                            edge0 = edge
                            idx += 1
                            continue
                    elif idx == 2 and tup != None:    
                        bolt_set2(tup, cpt, edge, pm)  # 単一（長穴）ボルト配置
                        #flg_tup = 1
                        #return hole_list2   # 空リスト
                        idx = 1
                        continue

                    hole_list2.append(holes)

                    idx += 1
                    if idx == 3:    #2個選択済
                        menu2 = False
                        break   # menu2を抜ける

        except:     # 他のコマンド選択された場合などでもエラー回避
            print('\n\n  エッジが選択されませんでした\007')
            menu2 = False
            #return None
            #continue

    #part_num = idx - 1

    #print(len(hole_list2), hole_list2)   #!!!
    
    return hole_list2  # 2list

#end of menu_edge


#-----------------------------------------------------------------------------
#   特定のタップに対するキリ穴検索関数　検索アルゴリズム改良(2) 同軸方式+近所  2020.8.21 追記
#-----------------------------------------------------------------------------
def get_blt_cp2(tp, kr_list):
    kr00 = []
    # 同軸もしくはキリ穴原点とタップ原点がgosa以下の相手を探す
    for kr in kr_list:
        tk00 = kr[2] - tp[2]
        if  LA.norm(tk00) < gosa:    # キリ穴原点=タップ原点 対応キリ穴発見!
            kr00 = kr
            break
        else:
            #tmp = tk00.dot(tp[4]) / LA.norm(tk00) / LA.norm(tp[4])
            #th00 =  np.arccos(np.clip(tmp, -1.0, 1.0))
            th00 = myv.vvth(tp[4], tk00)
            if LA.norm(tk00) * np.sin(th00) < gosa:  # 同軸キリ穴発見!
                kr00 = kr
                break

    # 無ければNoneを返す    
    if len(kr00) == 0:
        print(' キリ穴が発見できませんでした')  #!!!
        return None

    #print(kr00)
    # タップのボルト挿入側を判定
    if LA.norm(tp[2] - kr00[2]) < LA.norm(tp[3] - kr00[2]):
        tp0 = tp[2]
        tp_vec0 = -tp[4]    #単位ベクトル
    else: 
        tp0 = tp[3]   # 小さい(近い)方が tp0
        tp_vec0 = tp[4]
    l_tp_vec0 = LA.norm(tp_vec0)

    # キリ穴のボルト挿入側を判定
    if LA.norm(kr00[2] - tp0) < LA.norm(kr00[3] - tp0):
        kr0 = kr00[3]
    else: kr0 = kr00[2]   # 大きい(離れている)方が kr0
   
    # 反対面の同軸タップの除外
    #tmp = tp_vec0.dot(tp[4]) / l_tp_vec0 / LA.norm(tp[4])
    #th1 = np.arccos(np.clip(tmp, -1.0, 1.0))
    th1 = myv.vvth(tp[4], tp_vec0)
    
    if tp[5] == False and th1 < np.pi/2:      # 貫通してなくて、反対面のタップなら
        print('貫通してなくて、反対面のタップ') #!!!
        return None 

    # 適当なボルトサイズの選定
    tk0 = kr0 - tp0
    l_tk0 = LA.norm(tk0)

    #print(tp_vec0.dot(tk0), l_tp_vec0, l_tk0) 
    #tmp = tp_vec0.dot(tk0) / l_tp_vec0 / l_tk0
    #th = np.arccos(np.clip(tmp, -1.0, 1.0))    #
    th = myv.vvth(tk0, tp_vec0)
    
    tp_vec1 = tp_vec0 * l_tk0*np.cos(th) 
    blt_cp = tp0 + tp_vec1
    blt_len = LA.norm(tp_vec1) + tp[0] * blt_njkm
    blt_b = math.ceil((blt_njkm+0.5)*tp[0])    # ネジ部の長さ 切上
    blt_vec = tp_vec0


    #print(blt_len, blt_cp, blt_vec, blt_b, kr)
    #print(blt_len)
    #return blt_len, blt_cp, blt_vec, blt_b, kr00
    return blt_len, blt_cp, blt_vec, blt_b, kr00
                

#--------------------------------------------------------------------
# 配置ボルトのリスト作成２  相手穴検索を相手部品に限定 2022.6.12改良
#--------------------------------------------------------------------
def get_bolt_list(hole_list2, bolt_spec): # 
    if len(hole_list2) == 1: #割締め部品
        wari = True
    else:  # 2部品締結
        wari = False

    #print(len(hole_list2),wari)  #!!!
    #kr_itm = hole_list[0]       # キリ穴部品のキリ穴情報
    #tp_itm = hole_list[1]       # タップ部品のタップ穴情報
    kr_list = [[],[]]     #キリ穴情報リスト[i][d,h,v_cp0,v_cp1,v_vec]
    tp_list = [[],[]]     #タップ情報リスト
    bolt_list = []   # 配置ボルトリスト[i][blt_size,blt_len,blt_cp,blt_vec,blt_type,blt_isw,blt_hw]
                  #                    サイズ,長さ,配置位置,方向,種類,オプション   
                  #                                             hex,cs 0:のみ 1:SW付 2:SW+平
    blt_type =  bolt_spec[0];  blt_zai = bolt_spec[1]; blt_sw = bolt_spec[2]
    blt_hw =  bolt_spec[3]; # blt_njkm = bolt_spec[4]; blt_tmjm = bolt_spec[5]
  
    # キリ穴とタップ情報の振り分け、取得

    i = 0
    for hl in hole_list2:
        #print(hl)   #!!!
        for hole in hl:

            #print(hole.name); input()
            if hole.name.find("キリ穴") > -1 or hole.name.find("レーザー") > -1:  
                d = hole.param_list[0].value  # 穴径 float
                h = hole.param_list[1].value  # 穴深さ float
        
                v_cp0 = np.array([hole.cs.org.x, hole.cs.org.y, hole.cs.org.z]) #穴中心
                v_vec = np.array([hole.cs.zvec.x, hole.cs.zvec.y, hole.cs.zvec.z])  # 穴方向単位ベクトル
                v_cp1 = v_cp0 + h * v_vec     # 穴底中心
                k_z = "k"                     # キリ穴
                tmp = [d,h,v_cp0,v_cp1,v_vec,k_z]
                #print(tmp)
                kr_list[i].append(tmp)

            elif hole.name.find("ざぐり穴") > -1:
                d = hole.param_list[1].value  # 穴径 float
                h = hole.param_list[3].value - hole.param_list[2].value  # キリ穴部深さ float
                hz = hole.param_list[2].value  # ザグリ深さ float
        
                v_vec = np.array([hole.cs.zvec.x, hole.cs.zvec.y, hole.cs.zvec.z])  # 穴方向単位ベクトル
                tmp0 = np.array([hole.cs.org.x, hole.cs.org.y, hole.cs.org.z]) #穴中心
                v_cp0 = tmp0 + hz * v_vec
                v_cp1 = v_cp0 + h * v_vec     # 穴底中心
                k_z = "z"                     # ザグリ穴
                tmp = [d,h,v_cp0,v_cp1,v_vec,k_z]     # 0:穴径、1:深さ、2:穴中心、3:穴底中心、4:穴方向、5:穴種類
                #print(tmp)
                kr_list[i].append(tmp)  

            elif hole.name.find("タップ穴") > -1:  
                d = hole.param_list[0].value  # 外径  float
                h = hole.param_list[2].value  # 穴深さ float

                kantu =  hole.penetrate       # 貫通属性　True False  2020.8.20追記
      
                v_cp0 = np.array([hole.cs.org.x, hole.cs.org.y, hole.cs.org.z]) #穴中心
                v_vec = np.array([hole.cs.zvec.x, hole.cs.zvec.y, hole.cs.zvec.z]) # 穴方向
                v_cp1 = v_cp0 + h * v_vec     # 穴底中心
                tmp = [d,h,v_cp0,v_cp1,v_vec, kantu]
                #print(tmp)
                tp_list[i].append(tmp)
                
        i += 1


            #print(len(tp_list))
    if len(kr_list[0]) == 0 and len(kr_list[1]) == 0:
        print('キリ穴がありません\007')
        return None
    if len(tp_list[0]) == 0 and len(tp_list[1]) == 0:
        print('タップ穴がありません\007')
        return None
  
    #-------------------------------------
    #     タップに対応するキリ穴の検索
    #-------------------------------------
    #flg_blt = 0
    cnt = 0; i = 0
    
    for tmp_list in tp_list:
        d, j = divmod(i+1, 2)   # i=0:j=1  i=1:j=0  
        #for tp in tp_list:
        #print(i, j) #!!!
        for tp in tmp_list:
            blt_size = tp[0]      #ボルトサイズ float
            #tmp = get_blt_cp(tp, kr_list)    # tapに対応したキリ穴を検索
            if wari:
                tmp = get_blt_cp2(tp, kr_list[i])    # tapに対応したキリ穴を検索　2020.8.21 追記
            else:
                tmp = get_blt_cp2(tp, kr_list[j])    # tapに対応したキリ穴を検索　2020.8.21 追記
            if tmp == None:
                print('対応するキリ穴がありません\007')
                continue
            else:
                blt_len, blt_cp, blt_vec, blt_b, kr = tmp

            if abs(blt_vec[0]) < 0.001: blt_vec[0] = 0.0   # この操作をしないとボルトが反転する
            if abs(blt_vec[1]) < 0.001: blt_vec[1] = 0.0
            if abs(blt_vec[2]) < 0.001: blt_vec[2] = 0.0

            if kr[5] == "z": blt_type = "cs"
            #else: blt_type = blt_type0
            #input(blt_type)
            bolt_list.append([blt_size, blt_len, blt_cp, blt_vec, blt_type, blt_sw, blt_hw, blt_b])
                      # Bolt_size 長さ　中心座標  配置方向Vec  hex,cs  有,無    有,無
        i += 1

        # float     float vec
        #print(blt_vec)
    #print(bolt_list)
    #print("Count=" + str(cnt))
    return bolt_list
# end of get_bolt_list


# 2023.7.8 追加   size_list[0,1..]


#***************************************************
# JIS部品(ボルト)をパーツ化して配置する。
#***************************************************

def bolt_set(bolt_list): #, bolt_spec, fn_dic, lj):
  layer = False   # 入力レイヤに作成する。
  mode = SxFileMachinePart.MODE_PART
  #bolt_exist= 0    # ボルト存在確認 Flag
  #print(bolt_list)  ####
  bolt_spec = get_bolt_spec()
  blt_type= bolt_spec[0]; blt_zai= bolt_spec[1]; blt_sw= bolt_spec[2]
  blt_hw= bolt_spec[3];  #blt_njkm = bolt_spec[4]; blt_tmjm = bolt_spec[5]

  for blt in  bolt_list:
    bolt_exist= 0    # ボルト存在確認 Flag
    flg1= 0   # 第一候補選択
    #-------------------------------
    # 各ボルトの設定ファイル名取得
    #-------------------------------
    ssb= 'M'+str(int(blt[0]))
    if blt[4] == "hex":
      if blt_zai == "SUS":
        flg= 0
        #print(ssb, len(size_hex_sus), size_hex_sus)
        for i in range(len(size_hex_sus)):
          if ssb in size_hex_sus[i]:  # sizeがあるなら
            fn_blt = fn_blt_hex_sus[i] 
            fn_sw = fn_sw_hex_sus[i]
            fn_hw = fn_hw_hex_sus[i]
            flg= 1; break
        if flg== 0: # sizeがない
            print('\n\n     1.呼び %s は登録されていません\007' % ssb)
            continue

      elif  blt_zai == "SS":
        flg= 0
        #print(ssb, size_hex_ss[0])
        for i in range(len(size_hex_ss)):
          if ssb in size_hex_ss[i]:  # sizeがあるなら
            fn_blt = fn_blt_hex_ss[i] 
            fn_sw = fn_sw_hex_ss[i]
            fn_hw = fn_hw_hex_ss[i]
            flg= 1; break
        if flg== 0: # sizeがない
            print('\n\n     2.呼び %s は登録されていません\007' % ssb)
            continue
        
    elif  blt[4] == "cs":
      if blt_zai == "SUS":
        flg= 0
        for i in range(len(size_cs_sus)):
          if ssb in size_cs_sus[i]:  # sizeがあるなら
            fn_blt = fn_blt_cs_sus[i] 
            fn_sw = fn_sw_cs_sus[i]
            fn_hw = fn_hw_cs_sus[i]
            flg= 1; break
        if flg== 0: # sizeがない
            print('\n\n     3.呼び %s は登録されていません\007' % ssb)
            continue
          
      elif blt_zai == "SS":        
        flg= 0
        for i in range(len(size_cs_ss)):
          if ssb in size_cs_ss[i]:  # sizeがあるなら
            fn_blt = fn_blt_cs_ss[i] 
            fn_sw = fn_sw_cs_ss[i]
            fn_hw = fn_hw_cs_ss[i]
            flg= 1; break
        if flg== 0: # sizeがない
            print('\n\n     4.呼び %s は登録されていません\007' % ssb)
            continue
   
    if i!= 0:   # 第一候補以外が選択された
        print('     第 %d 候補が選択されました' % int(i+1))
        flg1= 1
    #------------------------------------
    #   parametor pattern の設定
    #------------------------------------
    #print(fn_hw)
    blt_parame = SxFileMachinePart(fn_blt)      # Bolt File パラメータ
    sw_parame = SxFileMachinePart(fn_sw)      # Spring Washer パラメータ
    hw_parame = SxFileMachinePart(fn_hw)      # 平 Washer パラメータ
  
    # 呼び名のリストを取得する。
    blt_pattern = blt_parame.getPatternNames()     # M3,M4...リスト
    sw_pattern = sw_parame.getPatternNames()     # M3,M4...リスト
    hw_pattern = hw_parame.getPatternNames()     # M3,M4...リスト
    #print(hw_pattern)   ####
    #------------------------------------
    # バネワッシャの辞書  
    #------------------------------------
    sw_dic = {}
    #index = sw_parame.getVarIndex("dd")  # パラメータdd(呼び径)のindexを得る
    for idx in range(len(sw_pattern)):    # 登録されてる全てのsw
      values = sw_parame.getVarValues(sw_pattern[idx])   # idxでボルトサイズ(M3,M4..M16)がきまる。
                                                          # iCAD プルダウンメニューの並び順が idx
      sw_dic[values[3]] = [idx,values[1]]       # M3 idx=0 sw_dic[3] = [0,0.7] 
      #print(idx)
      #for val in values:
      #  print("  " + str(val))
      #print(sw_dic[values[1]])

    #------------------------------------
    # 平ワッシャの辞書
    #------------------------------------
    hw_dic = {}
    #index = hw_parame.getVarIndex("dd")  # パラメータdd(呼び径)のindexを得る
    for idx in range(len(hw_pattern)):    # 登録されてる全てのhw
      values = hw_parame.getVarValues(hw_pattern[idx])   # idxでボルトサイズ(M3,M4..M16)がきまる。
                                                          # iCAD プルダウンメニューの並び順が idx
      hw_dic[values[3]] = [idx,values[2]]       # hw_dic[3] = [0,0.5]
      #print(values[3])
      #print(hw_dic[values[3]])
    
    
    #------------------------------------
    # 配置座標、方向ベクトルを設定する。
    #------------------------------------
    pos = SxPos( blt[2][0], blt[2][1], blt[2][2])

    vx0 = np.array([1.0,0.0,0.0])
    vy0 = np.array([0.0,1.0,0.0])
    vy0m = np.array([0.0,-1.0,0.0])
    vz0 = np.array([0.0,0.0,1.0])
    #v1 = blt[3]
    tmp_v1 = blt[3]

    # 2022.6.13追記 これをしないとベクトルが直交しない
    v1 = [round(tmp_v1[0], 5), round(tmp_v1[1], 5), round(tmp_v1[2], 5)]
    
    if (v1 == vy0).all():
      #print('v1 == vy0')  
      v_vec = vy0
      xvec = vx0; zvec = vz0
    elif (v1 == vy0m).all():
    #elif v1[0] == vy0m[0] and v1[1] == vy0m[1] and v1[2] == vy0m[2]:
      #print('v1 == vy0m')  
      v_vec = vy0m
      xvec = vz0; zvec = vx0
    else: 
      #print('v1 == 任意')  
      v_vec0 = np.cross(vy0,v1)     # 法線ベクトル
      v_vec = v_vec0/LA.norm(v_vec0) # 単位法線ベクトル
      tmp = np.dot(vy0,v1)         # 内積 
      th = math.acos(tmp / LA.norm(vy0) / LA.norm(v1))
      xvec = myv.ewth(vx0,v_vec,th)
      zvec = myv.ewth(vz0,v_vec,th)
   
    #print('\n   xvec zvec表示\n')
    #print('v1[0]= %1.16f  v1[1]= %1.16f  v1[2]= %1.16f' % (v1[0], v1[1], v1[2]))  
    #print('vy0m[0]= %1.15f  vy0m[1]= %1.15f  vy0m[2]= %1.15f' % (vy0m[0], vy0m[1], vy0m[2]))  
    #print(v1, xvec, zvec)    #!!!

    xx = xvec[0];  xy = xvec[1]; xz = xvec[2] 
    zx = zvec[0];  zy = zvec[1]; zz = zvec[2] 
    
    xvec = SxVec( xx,xy,xz )    # z 軸周りのv[1,0,0]の回転ベクトル
    zvec = SxVec( zx,zy,zz )    # x 軸周りのv[0,0,1]の回転ベクトル
    
    #----------------------------------
    # ワッシャの配置、ボルト長さの決定
    #----------------------------------
    v_offset = np.array([0, 0, 0]) # sw,hw有時の移動ベクトル
    len_offset = 0
   
    washa = []      # 階層変更のためのワッシャリスト

    if blt[6] == "有":    # 平ワッシャ
      hw_yobi = blt[0]
      #print(hw_yobi)    ####
      try:
        hw_idx = hw_dic[hw_yobi][0]     # M3:0 M4:1...
        #hw_idx = sw_dic[hw_yobi][0]     # M3:0 M4:1...
        tmp = hw_parame.open(mode, hw_pattern[hw_idx], pos, zvec, xvec, layer)  # 平ワッシャ配置
      
        washa.append(tmp)     # 階層変更ワッシャ追加
      except:
        print('\n\n     5.呼び %s は登録されていません\007' % ssb)
        continue
        #return

      v_offset = hw_dic[hw_yobi][1]*blt[3]  # blt[3] = blt_vec [0.0,-1.0,0.0]

    if blt[5] == "有":    # ばねワッシャ
      sw_yobi = blt[0]
      try:
        sw_idx = sw_dic[sw_yobi][0]     # M3:0 M4:1...
        v_cp = blt[2] + v_offset
        x = v_cp[0];y = v_cp[1]; z = v_cp[2] 
        pos = SxPos( x,y,z )
        tmp = sw_parame.open(mode, sw_pattern[sw_idx], pos, zvec, xvec, layer)  # バネワッシャ配置

        washa.append(tmp)     # 階層変更ワッシャ追加
      except:
        print('\n\n     6.呼び %s は登録されていません\007' % ssb)
        continue
        #return

      v_offset = v_offset + sw_dic[sw_yobi][1]*blt[3]
   
    len_offset = LA.norm(v_offset)  # ワッシャの厚み分

    index = blt_parame.getVarIndex("d")  # パラメータd(直径)のindexを得る
    #bolt_exist= 0    # ボルト存在確認 Flag
    for idx in range(len(blt_pattern)):    # 登録されてる全てのボルト
      values = blt_parame.getVarValues(blt_pattern[idx])   # idxでボルトサイズ(M3,M4..M16)がきまる。
      #print(type(values[1]))                        # iCAD プルダウンメニューの並び順が idx
      #print(type(blt[0]))
      #print(values[1], blt[0])  ####

      if values[index] == blt[0]:         # blt_size == d なら float
        bolt_exist= 1
        #print(str(values[1]) + ":"+str(blt[0]))

        # ボルト長さ,ねじ長さの指定-----------------------------------------------------------------
        index = blt_parame.getVarIndex("l")
        blt_len = blt[1] + len_offset  # ワッシャの厚みを考慮した長さ
        
        #print(blt_len, blt[1], len_offset)  #!!!
        
        values[index] = lj[int(blt_len)]

        # 19.11.3 ネジ部の長さを変更-------------------------------------
        if blt[4] == "hex":     # 六角ボルトの場合    
          index_b = blt_parame.getVarIndex("b")
          index_x = blt_parame.getVarIndex("x") # x:不完全ネジ部長さ
          x = values[index_x]
          if blt[7] + x <= blt_len:   # xがmin以下でないなら 
            values[index_b] = blt[7]  # ネジ部の長さ
          else:
            values[index_b] = blt_len - x   
        elif blt[4] == "cs":    # CSボルトの場合
          index_lg = blt_parame.getVarIndex("lg")   #lg:不完全ネジ部長さ
          lg = values[index_lg]
          if blt[7] + lg <= blt_len:              # lg がmin以下でないなら 
            values[index_lg] = blt_len - blt[7]   # 不完全ネジ部長さ
          #else:
          #　values[index_lg] = blt_len - blt[7]
        #print(values[index])
        # パラメタの変更を反映する。
        blt_parame.setVarValues(blt_pattern[idx], values)

        break #continue
      
    if bolt_exist == 0:    # 一致するボルトがない場合
        print('\n\n     6.呼び %s が登録されていません\007' %  ssb)
        continue
        #return


    #------------------------------------
    #   ボルトの配置
    #------------------------------------
    v_cp = blt[2] + v_offset
    x = v_cp[0]; y = v_cp[1]; z = v_cp[2] 
    #print(v_cp)
    pos = SxPos(x,y,z)
    #print(blt_pattern[idx]) #!!!
    #print(zvec.x, zvec.y, zvec.z)
    #print(xvec.x, xvec.y, xvec.z)
    parent = blt_parame.open(mode, blt_pattern[idx], pos, zvec, xvec, layer)

    # 第一候補以外に色を付ける(灰色)
    if flg1== 1: 
        SxEnt.setColor([parent], SxSys.COLOR_GRAY)

    if len(washa) != 0 and v_blt.get():     # ワッシャ類があってCheck ButtonがTrueなら
      for part in washa:
        part.setParent(parent)  # 階層変更
    
    # 干渉チェック(要素周辺)
        seg = parent.checkCollision(-0.01, False)  
        # マイナス値の場合、指定クリアランス値以下の薄皮での干渉を無視。（近似計算）
        #seg = parent.checkCollision(0.0, True)  # 透明
        #seg = parent.checkCollision(0.1, False)
        if seg!= None:  # 干渉あり
            print('     干渉あり!!!')
            if v_chk.get(): # 干渉chk ONN
                # UNDOコマンドを実行します。
                SxSys.command(";UNDO", False)
                SxEnt.delete([parent])  # ok

#end of bolt_set()

def get_bolt_spec():
    global blt_njkm
    if v[0].get() == 0:     #種類   
        blt_type = 'hex'
    else: blt_type = 'cs'
    if v[1].get() == 0:     #材質
        blt_zai = 'SUS'
    else: blt_zai = 'SS'
    if v[2].get() == 0:     # バネワッシャ
        blt_sw = '有'
    else: blt_sw = '無'
    if v[3].get() == 0:     # 平ワッシャ
        blt_hw = '有'
    else: blt_hw = '無'

    # ねじ込み長さ係数    
    ss= cbx_v1.get()
    blt_njkm = float(ss)      # ねじ込み長さ係数
    #blt_tmjm = 0        # 共締め部品暑さ
    #print(blt_njkm) ####
    bolt_spec = [blt_type, blt_zai, blt_sw, blt_hw]
    
    return bolt_spec


def push_exe(): #実行ボタン 2022.6.12 同一部品除外
    bolt_spec = get_bolt_spec()
    
    loop = True

    while loop:
        # 穴情報取得-------------------------------------------------
        hole_list2 = menu_edge()   # 2list
        if len(hole_list2) == 0:
            print('\n  ボルト配置コマンド終了\n\007')
            break
        # ボルトリスト取得--------------------------------------------
        #if len(hole_list2) == 1: #割締め部品
        bolt_list = get_bolt_list(hole_list2, bolt_spec)
        #else:  # ２つの部品の締結
        #    tmp1 = get_bolt_list(hole_list2, bolt_spec)
        #    tmp2 = get_bolt_list(hole_list2, bolt_spec)
        #    bolt_list = tm1 + tmp2

        if bolt_list == None:
            continue

        # ボルト配置--------------------------------------------------
        bolt_set(bolt_list) #, bolt_spec, fn_dic, lj)
        print('\n  ボルト配置完了\n\007')
    #end of loop

    # 「穴属性」付加　20.10.3
    holatr()

def holatr():  # 「穴属性」付加　20.10.3
    ss=';BLOAD'
    ss=ss+';JLOD                                                                           '
    ss=ss+';LOD                                                                            '
    ss=ss+'.ZUMEN /キリ穴@3D                               /                               '
    ss=ss+'.DIR1 /C:\ICADSX\JISPARTS\JIS\\3D\\04_ねじ穴\キリ穴                      /        '
    #ss=ss+'.DIR1 /C:\\ICADSX\\JISPARTS\\USER\\01_SBY\\04_ねじ穴_1\\キリ穴             /        '
    #ss=ss+'.DIR1 /C:\ICADSX\JISPARTS\USER\\01_SBY\\04_ねじ穴_1\キリ穴               /        '
    ss=ss+'.DIR2 /                                                                /        '
    ss=ss+'.DIR3 /                                                                /        '
    ss=ss+'.DIR4 /                                                                /        '
    ss=ss+'.HAICHI /ｷﾘ穴_1.6                                /                              '
    ss=ss+'.ATR1 /33554432/                                                                '
    ss=ss+'.ATR2 /3       /                                                                '
    ss=ss+'.GENTENX /174.100969  /                                                         '
    ss=ss+'.GENTENY /40.31112    /                                                         '
    ss=ss+'.COMMENT /キリ穴_1.6                                      /                     '
    ss=ss+'.ATR3 /436207616   /                                                            '
    ss=ss+':                                                                               '
    ss=ss+'..HOLATR /1/                                                                    '
    ss=ss+'..HOLTEG /1/                                                                    '
    ss=ss+'..HOLNUM /1/                                                                    '
    ss=ss+'..NPARM1 /16777216/                                                             '
    ss=ss+'..PRTVER /1/                                                                    '
    ss=ss+'.YOBI /1.6                     /                                                '
    ss=ss+':                                                                               '
    ss=ss+'.PARAM /l                                               /                       '
    ss=ss+'.ATAI /2.5         /                                                            '
    ss=ss+':                                                                               '
    ss=ss+'.PARAM /d                                               /                       '
    ss=ss+'.ATAI /1.6         /                                                            '
    ss=ss+':                                                                               '
    ss=ss+'.PARAM /                                                /                       '
    ss=ss+'.ATAI /0           /                                                            '
    ss=ss+':                                                                               '
    ss=ss+'..PRTEND /1/                        '
    SxSys.command(ss,True)
    SxSys.command(';COPY3',False)   #False:Command mode
    

# 終了時の処理(画面サイズ,位置保存)
def on_closing():
    ss_geom = root.geometry() 
    mmd.save_geometry(save_fn_geom, ss_geom)
    root.destroy()


#------------------------------------------------------------
#   メインルーチン
#------------------------------------------------------------
# target_fn = 'icd_bolt_set.ini'  # 設定ファイル名  -> 引数渡しに変更
if len(sys.argv) < 2:   # 引数無し
    #if os.path.exists('C:\\ICADSX\\JISPARTS\\JIS\\キリ穴\\キリ穴.csv'):
    #    target_fn = 'icd_bolt_set.ini'  # 設定ファイル名  -> 引数渡しに変更
    if os.path.exists('C:\\ICADSX\\JISPARTS\\USER\\01_SBY\\04_ねじ穴_1\\キリ穴\\キリ穴.csv'):
        target_fn = 'icd_bolt_set_sby.ini'  # sny要設定ファイル名 
    else:
        target_fn = 'icd_bolt_setJ.ini'  # ICAD標準設定ファイル名 

    print(target_fn)
    
else:
    target_fn = sys.argv[1]     # target_fnを起動時に引数から読み取る

(fn_blt_hex_sus, fn_sw_hex_sus, fn_hw_hex_sus, fn_blt_hex_ss, fn_sw_hex_ss, fn_hw_hex_ss, \
     fn_blt_cs_sus, fn_sw_cs_sus, fn_hw_cs_sus, fn_blt_cs_ss, fn_sw_cs_ss, fn_hw_cs_ss) = get_ini(target_fn)

#fn_dic = get_ini(target_fn)     # ボルトファイル名読み込み
lj = set_jis_len()              # JIS規格ボルト長さの配列取得

#print(fn_dic)


#--------------------------------------------------
#   画面設定
#--------------------------------------------------
root = tk.Tk()
root.attributes("-topmost", True)    # 常に前面に表示
root.configure(bg='RoyalBlue4')
root.title('ボルトセット Ver.1.30')   #タイトル
#root.minsize(600,350)    # Windowの大きさ

# 画面サイズ、位置再現のための起動時の処理
fn = os.path.basename(__file__)     # get_place.py   
(tmp, ext) = os.path.splitext(fn)   # (get_place, py)
save_fn_geom = '%s.ini' % tmp
ss_geom = mmd.read_geometry(save_fn_geom)
if ss_geom == None:
    ss_geom = '280x120+1100+0'
    
root.geometry(ss_geom)
# blue  red  khaki3  cyan  white

#************************************
# ボタン等の部品配置
#************************************
btn_bcol = 'grey21'     #ボタン背景色
btn_tcol = 'white'      #ボタン文字色
label_bcol = 'khaki3'   #ラベル背景色
label_tcol = 'black'   #ラベル文字色
e_bcol = 'khaki4'
x00 = 10; y00 = 5
dx = 70;  dy = 25


#-----------------------------------------------
# Radiobutton Label Entry部品の配置   grid方式
#-----------------------------------------------
ss_l0 = ['種類', '材質', 'バネW','平W']
ss_l = [['六角ボルト', 'Cap Screw'],[ 'SUS', 'SS'],[ '有', '無'],[ '有', '無']]
v = [tk.IntVar(root, value = 0), tk.IntVar(root, value = 0),tk.IntVar(root, value = 0),tk.IntVar(root, value = 1)]
x = 0 ; y = 0 ; wd = 20
idx = 0
for ss in ss_l:
    #if 0 == idx:
    #v.append(tk.IntVar(root, value = 0))    #初期値設定
    f1 = ttk.LabelFrame(root, text = ss_l0[idx])        #frame作成
    for i in range(len(ss)):
        r = ttk.Radiobutton(f1, text = ss[i], value = i, variable = v[idx])
        #                 frame   見出し      ボタンNo  チェックの有無             
        r.pack()

    f1.grid(column=x, row=y, padx=5, pady=5)   
    #                      外側の横,縦の隙間を指定
    #f1.place(x=x00,y=y00+idx*dy)   
    idx += 1; x += 1

#px = 2; py = 2
y1= 75
#----------------------------------------------------------------
ss = '実行' ; x = 3 ; y +=1 # ボタン  
#----------------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_exe, bg=btn_bcol, fg=btn_tcol) #ボタンの作成
btn.place(x= 220, y= y1)
#btn.grid(column=x, row=y, padx=px, pady=py)

#---------------------------------------------------
ss = '一体化'   #  Check Button
#---------------------------------------------------
v_blt = tk.BooleanVar()   # 論理値型の変数 Instance
v_blt.set(True)          # bolt配下にワッシャ類配置 
cb_blt = tk.Checkbutton(root,text=ss,variable=v_blt)
cb_blt.place(x=85 , y= y1)
#cb_blt.place(x=150 , y= y1)
#cb_blt.grid(column=x, row=y, padx=px, pady=py)

#---------------------------------------------------
ss = '干渉chk'  #  Check Button
#---------------------------------------------------
v_chk = tk.BooleanVar()   # 論理値型の変数 Instance
v_chk.set(False)          # 
#v_chk.set(True)          # 
cb_chk = tk.Checkbutton(root,text=ss,variable=v_chk)
cb_chk.place(x=5, y= y1)
#cb_chk.place(x=70, y= y1)

#---------------------------------------------------
ss = 'ねじ込深係数'
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x= 5, y= 105)
#entry0= tk.Entry(root, width=4,justify="right",bg=e_bcol)
#entry0.place(x=80, y= y1)
#entry0.insert(tk.END,str(blt_njkm))

cbx_v1 = tk.StringVar()
cbx = ttk.Combobox(root, textvariable=cbx_v1, width= 4)
#cbx.bind('<<ComboboxSelected>>', cb_selected)
cbx['values']=('0.8','1.0','1.2','1.5','2.0')
cbx.current(2)   # cb初期設定  'Foo'  Default
cbx.place(x=80, y= 105)    



root.protocol("WM_DELETE_WINDOW", on_closing)   #終了時処理
root.mainloop()

